/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */
swarm.workflow={

    init: function(options) {
        $(document).on('click.disabled','.disabled',function(e) {
            e.preventDefault();
            e.stopPropagation();
        });
        swarm.workflow.defineTemplates({});
        swarm.workflow.load();
        $(document).on('swarm-login', function (e) {
            swarm.workflow.load();
        });
        $('a').tooltip(); // Enables boostrap tooltip features
        $('#add-workflow-button').on('click',function(e) {
            swarm.workflow.showWorkflowModal({
                title:swarm.te('Add Workflow'),
                mutable:true
            });
        });
        $('#workflow-modal-content').on('click.save','.btn-save', function(e) {
            e.preventDefault();
            var form = $(this).closest('form'),
                id = form.data('workflow-id'),
                url = '/api/v9/workflows'+(id?"/"+id:"");
            swarm.form.rest(id?'PATCH':'POST',url, form,
                // Callback(success handler)
                function(response, form) {
                    if (response.workflow) {
                        var savedMessage = $.templates(
                            '<div class="swarm-alert alert alert-success" id="inner-message">{{>message}}</div>'
                        ).render({message: swarm.te('Workflow Saved')});
                        $('.modal-body .messages').append(savedMessage);
                        // Reload tabs
                        swarm.workflow.load();
                        // close the form on completion
                        setTimeout(function() {
                            form.find('button.close').click();
                        },500);
                    }
                },
                // Error node
                $('#workflow-modal-content .modal-body')
            );
        });
        $('#workflow-modal-content').on('click.rule','.control-group-workflow-rules .btn-group li a', function(e) {
            var choice = $(this),
                group  = choice.closest('.btn-group');
                group.find('span.current').text(choice.text());
                group.find('input').val(choice.data('value'));
        });
        $('.workflow-cells').on('click.workflow','.link.workflow-name a',function(e) {
            var link = $(this),
                cell = $(this).closest('.workflow-cell');
           e.preventDefault();
           swarm.workflow.showWorkflowModal($.extend({title:link.text()},cell.data()));
        });
        $('.btn-search').on('click',function() {
            $('div.filter-all').remove();
            $('.workflow-cell').toggleClass('filtered',true).each(function() {
                var searchTerm = $('.search .input-large').val().toLowerCase(),
                    workflow = $(this);
                if (workflow.find('a.name').text().toLowerCase().indexOf(searchTerm) !== -1 ||
                    workflow.find('.description').text().toLowerCase().indexOf(searchTerm) !== -1) {
                    $(this).toggleClass('filtered',false);
                }
            });
            if ($('.workflow-cell:visible').length === 0) {
                $('.workflow-cells').prepend($('<div>').addClass('filter-all').text(swarm.te('No matching workflows')));
            }
        });
        $('.workflows .search input').on(
            'keyup',
            function(e) {
                // early exit if not enter key
                var code = (e.keyCode || e.which);
                if (e.type === 'keyup' && code !== 13) {
                    return;
                }
                $('.btn-search').trigger('click');
            }
        );
        $('#workflow-modal-content').on('click.delete','#btn-delete', function(e) {
            e.preventDefault();
            var button = $(this),
                form = button.closest('form'),
                url = '/api/v9/workflows/' + form.data('workflow-id');

            setTimeout(function () {
                var confirm = swarm.tooltip.showConfirm(button, {
                    placement: 'top',
                    content: swarm.te('Delete this workflow?'),
                    classes: 'popover-workflow',
                    buttons: [
                        '<button type="button" class="btn btn-primary btn-confirm">' + swarm.te('Delete') + '</button>',
                        '<button type="button" class="btn btn-cancel">' + swarm.te('Cancel') + '</button>'
                    ]
                });

                // wire up cancel button
                confirm.tip().on('click', '.btn-cancel', function () {
                    confirm.destroy();
                });

                confirm.tip().on('click', '.btn-confirm', function () {
                    // disable buttons when the delete is in progress
                    swarm.form.disableButton(confirm.tip().find('.btn-confirm'));
                    confirm.tip().find('.buttons .btn').prop('disabled', true);

                    swarm.form.restDelete(url, form,
                        // Callback(success handler)
                        function (response, form) {
                            confirm.destroy();
                            if (response.isValid) {
                                var deletedMessage = $.templates(
                                    '<div class="swarm-alert alert alert-success" id="inner-message">{{>message}}</div>'
                                ).render({message: swarm.te('Workflow Deleted')});
                                $('.modal-body .messages').append(deletedMessage);
                                // Reload tabs
                                swarm.workflow.load();
                                // close the form on completion
                                setTimeout(function () {
                                    form.find('button.close').click();
                                }, 500);
                            } else {
                                var errorConfirm = swarm.tooltip.showConfirm(button, {
                                    placement: 'top',
                                    content: response.error,
                                    buttons: [
                                        '<button type="button" class="btn btn-primary">' + swarm.te('Ok') + '</button>'
                                    ]
                                });
                                errorConfirm.tip().on('click', '.btn', function () {
                                    errorConfirm.destroy();
                                });
                            }
                        },
                        // Error node
                        $('#workflow-modal-content .modal-body')
                    );
                });
            }, 5);
        });
    },

    load: function() {
        $('.workflow-cells').html(swarm.t('Loading...'));
        $.ajax({
            url: '/api/v9/workflows',
            dataType: 'json',
            success: function(data) {
                var authenticatedUser = swarm.user.getAuthenticatedUser();
                var mine = $('div#my-workflows .workflow-cells').html(authenticatedUser!==null?'': ('<div class="muted">'
                    + swarm.te("My workflows will only be populated once you have logged in.")
                    + '<a href="/login/" onclick="swarm.user.login(); return false;"> ' + swarm.te("Log in") + '</a> '
                    + swarm.te("now.")));
                var all = $('div#all-workflows .workflow-cells').html('');
                var ownerIconTitle = swarm.t('You are an owner');
                var workflow;
                $.each(data.workflows.sort(
                    function(w1, w2) {
                        var w1lower = w1.name.toLowerCase(),
                            w2lower = w2.name.toLowerCase();
                            return w1lower < w2lower ? -1 : w1lower > w2lower ? 1 : 0;
                        }),
                    function() {
                        workflow = $($.templates(swarm.workflow.summary).render({workflow: this, ownerIconTitle: ownerIconTitle}));
                        var isSuper = $('body').hasClass('super');
                        $(workflow).data({'workflow': this, 'mutable': false});
                        if (null !== authenticatedUser &&
                            $([authenticatedUser.id].concat(Object.keys(authenticatedUser.groups || {})))
                                .filter(this.owners.map(function(owner) {
                                    return owner.replace(/^swarm-group-/,'');
                                })).length > 0) {
                            workflow.addClass('owner').data('mutable', true).appendTo(all).clone().appendTo(mine).data({
                                'workflow': this,
                                'mutable': true
                            });
                        } else if (this.shared || isSuper) {
                            workflow.appendTo(all).data({
                                'mutable': isSuper
                            });
                        }
                    }
                );
                $('.workflow-cells').each(function() {
                    if (!this.hasChildNodes()) {
                        $(this).html(swarm.te('No Workflows to show.'));
                    }
                });
                $('.workflow-cell .description').expander({slicePoint: 100});
            }
        });
    },

    /**
     * Display a modal form, prepopulated with any workflow data provided in data.workflow.
     * @param data - object
     */
    showWorkflowModal: function(data) {
        var content  = $('#workflow-modal-content'),
            workflow = data.workflow || {owners: [swarm.user.getAuthenticatedUser().id]},
            mutable  = data.mutable||false,
            onSubmit = workflow.on_submit || null,
            endRules = workflow.end_rules || null,
            defaults = swarm.workflow.defaults.workflow_rules,
            rules    = [
            {
                description: swarm.te('On commit without a review'),
                target: 'submit_without_review_action',
                name: 'on_submit[without_review]',
                rule: (onSubmit && onSubmit.without_review) ? onSubmit.without_review.rule : defaults.on_submit.without_review.rule,
                defaultErr: defaults.on_submit.without_review.rule,
                options: [
                    {value: 'no_checking', text: swarm.te('Allow')},
                    {value: 'auto_create', text: swarm.te('Create a review')},
                    {value: 'reject',      text: swarm.te('Reject')}
                ]
            },
            {
                description: swarm.te('On commit with a review'),
                target: 'submit_with_review_action',
                name: 'on_submit[with_review]',
                rule: (onSubmit && onSubmit.with_review) ? onSubmit.with_review.rule : defaults.on_submit.with_review.rule,
                defaultErr: defaults.on_submit.with_review.rule,
                options: [
                    {value: 'no_checking', text: swarm.te('Allow')},
                    {value: 'approved',    text: swarm.te('Reject unless approved')}
                ]
            },
            {
                description: swarm.te('On update of a review in an end state'),
                target: 'submit_end_rule_action',
                name: 'end_rules[update]',
                rule: (endRules && endRules.update) ? endRules.update.rule : defaults.end_rules.update.rule,
                defaultErr: defaults.end_rules.update.rule,
                options: [
                    {value: 'no_checking', text: swarm.te('Allow')},
                    {value: 'no_revision', text: swarm.te('Reject')}
                ]
            },
            {
                description: swarm.te('Count votes up from'),
                target: 'counted_votes_action',
                name: 'counted_votes',
                rule: workflow.counted_votes ? workflow.counted_votes.rule : defaults.counted_votes.rule,
                defaultErr: defaults.counted_votes.rule,
                options: [
                    {value: 'anyone',  text: swarm.te('Anyone')},
                    {value: 'members', text: swarm.te('Members')}
                ]
            },
            {
                description: swarm.te('Automatically approve reviews'),
                target: 'submit_auto_approve_action',
                name: 'auto_approve',
                rule: workflow.auto_approve ? workflow.auto_approve.rule : defaults.auto_approve.rule,
                defaultErr: defaults.auto_approve.rule,
                options: [
                    {value: 'never', text: swarm.te('Never')},
                    {value: 'votes', text: swarm.te('Based on vote count')}
                ]
            }
        ];
        content.html($.templates(swarm.workflow.formTemplate).render({
            title:data.title||swarm.t('Workflow'),
            mutable: mutable,
            workflow: workflow,
            rules: rules
        }));
        content.find('#owners').userMultiPicker({
            itemsContainer: content.find('.owners-list'),
            selected:        workflow.owners.filter(function(owner) { return -1 === owner.indexOf('swarm-group-');}),
            inputName:       'owners',
            enableGroups:    true,
            required:        true,
            groupInputName:  'owners',
            useGroupKeys:    true,
            excludeProjects: true,
            disabled:        content.find('#owners').hasClass('disabled'),
            selectedGroups:  workflow.owners.filter(function(owner) { return -1 !== owner.indexOf('swarm-group-');})
        });
        $.each(content.find('.control-group-workflow-rules .btn-group.rule'),function(){
            var rule = $(this);
            rule.find('span.current').text(rule.find('li a[data-value="'+rule.find('input').val()+'"]').text()||swarm.t('Please choose an option'));
        });
        var workflowId = workflow.id;
        // Fetch the projects for this workflow.
        $.ajax({
                url: '/api/v9/projects?workflow='+workflowId,
                success: function(data) {
                    if (data.projects.length > 0) {
                        $('#affected-projects').empty()
                            .append(
                                $('<a href="#project-list" data-toggle="collapse" data-target="#project-list">')
                                    .addClass("project-count collapsed")
                                    .append($('<i>').addClass("icon icon-chevron-down"))
                                    .append($('<span>')
                                        .addClass('collapse-title')
                                        .text(
                                            swarm.tpe(
                                                '%d project uses this workflow',
                                                '%d projects use this workflow',
                                                data.totalCount
                                            ))))
                            .append(
                                $('<div>').attr('id', 'project-list').addClass('collapse')
                                    .append(
                                        $.templates(swarm.workflow.affectedProjectsTemplate)
                                            .render({projects:data.projects, workflowId: workflowId})));
                    } else if (data.totalCount) {
                        $('#affected-projects p').removeAttr('class').text(
                            swarm.tpe(
                                '%d project uses this workflow',
                                '%d projects use this workflow',
                                data.totalCount
                            ));
                    } else {
                        $('#btn-delete').prop('disabled', !mutable);
                        $('.tool-tip').removeAttr('title');
                        $('#affected-projects p').removeAttr('class').text(
                            swarm.t(
                                'No projects use this workflow'
                            ));
                    }
                }
        });
    },

    workflowDropdown: function(options) {
        function addEvents(localStorageKey, prefix) {
            function clearWorkflowListStyling(list) {
                list.find("a.btn-filter").each(function() {
                    this.innerHTML=$(this).data('short-label');
                    $(this).show();
                });
            }

            $('.' + prefix + '-workflow-filter').on('click', '.input-filter', function(e) {
                e.stopPropagation();
            });

            $('.' + prefix + '-workflow-filter .input-filter').each(function() {
                var $input = $(this),
                    workflowControl = $('#' + prefix + '-workflow-control'),
                    list = workflowControl.find('.dropdown-menu li.list-item');

                $input.on('input change', function () {
                    $input.siblings('.clear').hide();

                    if (!$input.val()) {
                        // if value is empty - show all and clear styling
                        clearWorkflowListStyling(list, workflowControl);
                    } else {
                        $input.siblings('.clear').show();
                        list.find("a.btn-filter").each(function() {
                            $(this).hide();
                        });
                        list.find("a.btn-filter:dataLabelContains('" + $input.val() + "')").each(function() {
                            var workflowName = $(this).data('short-label');
                            var reg = new RegExp($input.val(), 'gi');
                            this.innerHTML=workflowName.replace(reg, "<b>$&</b>");
                            $(this).show();
                        });
                    }

                }).change();

                $input.siblings('.clear').on('click', function (e) {
                    $input.val('').data('filter-value', '').change();
                    e.stopPropagation();
                    clearWorkflowListStyling(list, workflowControl);
                });

                // handling 'enter' and 'esc' keys and arrow-down event on the input box
                $input.on('keydown', function(e) {
                    var listItems = workflowControl.find('.dropdown-menu');
                    if(e.keyCode === 13) {
                        $(document).trigger('click.dropdown.data-api');
                        e.preventDefault();
                    } else if(e.keyCode === 27) {
                        $input.val('');
                        $(document).trigger('click.dropdown.data-api');
                        e.preventDefault();
                    } else if (e.keyCode === 40) {
                        e.stopImmediatePropagation();

                        workflowControl.find('ul.dropdown-menu').children().each(function(){
                            $(this).removeClass('active');
                        });
                        var itemID = listItems.find('li.list-item a:visible').first().closest('li').attr('id');
                        $('#' + itemID).find('a').focus().addClass('active');
                    }

                });

                // wire up navigation - as default navigation doesnt like hidden elements we need to find visible ones ourselves
                workflowControl.find('ul.dropdown-menu').on('keydown', function(e){
                    if (e.keyCode === 40) {
                        e.stopImmediatePropagation();
                        e.preventDefault();
                        // find current visible element and find id of a next visible element
                        var nextElementID = $(document.activeElement).closest('li').nextAll('li').find('a:visible,input').first().closest('li').attr('id');
                        if (nextElementID) {
                            $(document.activeElement).removeClass('active');
                            $('#' + prefix + '-' + nextElementID).find('a,input').focus().addClass('active');
                        }

                    } else if (e.keyCode === 38) {
                        e.stopImmediatePropagation();
                        e.preventDefault();
                        // find current visible element and find id of a previous visible element
                        var previousElementID = $(document.activeElement).closest('li').prevAll('li').children('a:visible,input').first().closest('li').attr('id');
                        if (previousElementID) {
                            $(document.activeElement).removeClass('active');
                            $('#' + prefix + '-' + previousElementID).find('a,input').focus().addClass('active');
                        } else {
                            // if we have not found an element at this level - try to go one higher
                            previousElementID = $(document.activeElement).closest('ul').prevAll('li').children('a:visible,input').first().closest('li').attr('id');
                            $(document.activeElement).removeClass('active');
                            $('#' + prefix + '-' + previousElementID).find('a,input').focus().addClass('active');
                        }
                    }
                });

                workflowControl.find('.dropdown-menu li:has(> a) a').on('click', function(){
                    localStorage.setItem(localStorageKey, $(this).parent().attr('id'));

                    clearWorkflowListStyling(list, workflowControl);
                    var idElement = $("#" + localStorage.getItem(localStorageKey) + " a");
                    $input.val(idElement.data('filter-value')).change();
                    $('#' + prefix + '-workflow').val(localStorage.getItem(localStorageKey).replace(prefix + '-', ''));
                    // if input is empty set filter value to groups default
                    var workflowValue = !$input.val() ? workflowControl.find('.default') : idElement;
                    if (!$(workflowValue).closest('.dropdown-menu').length) {
                        $(workflowValue).toggleClass('active');
                    } else if ($(workflowValue).is('input')) {
                        $(workflowValue).data('filter-value', $(workflowValue).val());
                        workflowControl.find('.text').text($(workflowValue).val());
                    } else {
                        $(workflowValue).addClass('active');
                        workflowControl.find('.text').text($(workflowValue).data('short-label') || $(workflowValue).html());
                        workflowControl.find('.input-filter').siblings('.clear').trigger('click');
                    }
                });
            });
        }

        function defineTemplates() {
            swarm.workflow.dropdown =
                '<div id="{{:prefix}}-workflow-control" class="" data-filter-key="workflow">\n'+
                '    <button id="btn-project-workflow" type="button" class="btn btn-workflow dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" title="">\n'+
                '        <span class="text workflow-label"></span>\n' +
                '        <span class="caret"></span>\n'+
                '    </button>\n'+
                '    <ul class="dropdown-menu workflow-filter" aria-label="' + swarm.te("Workflows") +'">\n'+
                '        <li id="{{:prefix}}-no-workflow">\n'+
                '            <a href="#" class="btn-filter default" data-filter-value="{{:noneSelected}}">{{:noneSelected}}</a>\n'+
                '        </li>\n'+
                '        <li class="divider"></li>\n'+
                '        <li class="{{:prefix}}-workflow-filter">\n'+
                '            <input class="input-filter" data-filter-key="workflow" type="text" placeholder="'+swarm.te("Workflows")+'">\n'+
                '            <button type="button" class="clear" style=""><span>x</span></button>\n'+
                '        </li>\n'+
                '        <li class="divider"></li>\n'+
                '    </ul>\n'+
                '</div>';
            swarm.workflow.dropdownmenuitem =
                '<li class="list-item" id="{{:prefix}}-{{:workflow.id}}">\n'+
                '    <a href="#" class="btn-filter btn-filter-workflow" data-filter-key="workflow" data-filter-value="{{:prefix}}-{{:workflow.id}}" data-short-label="{{:workflow.name}}">{{:workflow.name}}</a>\n'+
                '</li>';
        }

        function init() {
            var parent = options.parent,
                prefix = options.prefix,
                workflows = options.workflows,
                currentWorkflowId = options.currentWorkflowId || -1,
                localStorageKey = options.localStorageKey,
                noneSelected = options.noneSelected || swarm.te('No workflow');
            defineTemplates();

            var workflowDropdown       = $($.templates(swarm.workflow.dropdown).render({prefix: prefix, noneSelected: noneSelected})),
                authenticatedUser      = swarm.user.getAuthenticatedUser(),
                isSuper                = $('body').hasClass('super'),
                menu                   = workflowDropdown.find('.dropdown-menu'),
                workflowControlClasses = parent.find('#' + prefix + '-workflow-control').attr("class");

            parent.find('#' + prefix + '-workflow-control').replaceWith(workflowDropdown);
            // preserve classes on the element being replaced
            workflowDropdown.addClass(workflowControlClasses);
            $.each(workflows.sort(
                function(w1, w2) {
                    var w1lower = w1.name.toLowerCase(),
                        w2lower = w2.name.toLowerCase();
                    return w1lower < w2lower ? -1 : w1lower > w2lower ? 1 : 0;
                }),
                function(index, value) {
                    // We want to include: All for super, any shared, any owned and also any that may no longer be
                    // shared or owned by where at the time the choice was made before
                    if (isSuper || value.shared || (parseInt(currentWorkflowId, 10) === value.id) ||(null !== authenticatedUser &&
                        $([authenticatedUser.id].concat(Object.keys(authenticatedUser.groups || {})))
                            .filter(this.owners.map(function(owner) {
                                return owner.replace(/^swarm-group-/,'');
                            })).length > 0)) {
                        var item = $($.templates(swarm.workflow.dropdownmenuitem).render({prefix: prefix, workflow: value}));
                        menu.append(item);
                    }
                });
            var defaultWorkflowValue   = $("#" + prefix + "-no-workflow a"),
                currentWorkflow = currentWorkflowId !== -1 ? $("#" + prefix + '-' + currentWorkflowId + " a") : defaultWorkflowValue;
            workflowDropdown.find('.text').text($(currentWorkflow).data('short-label') || $(currentWorkflow).html());
            addEvents(localStorageKey, prefix);
            workflowDropdown.find('button').prop('disabled', false);
            return workflowDropdown;
        }
        return init();
    },

    defineTemplates: function(options) {
        swarm.workflow.summary =
            '<div class="workflow-cell pad1 workflow-cell-{{:workflow.id}}">\n' +
            '  <div class="content pad2"><span class="workflow-name link"><i class="swarm-icon icon-workflow-owner mutable-only" title="{{>ownerIconTitle}}"></i><a data-toggle="modal" data-target="#workflow-modal-content" href="#" class="force-wrap name" data-original-title="" title="">{{>workflow.name}}</a></span>\n' +
            '    <p class="force-wrap description muted"><small>{{>workflow.description}}</small></p>\n' +
            '  </div>\n' +
            '</div>\n';
        swarm.workflow.formTemplate =
            '<form method="post" class="modal-form"{{if workflow.id}} data-workflow-id="{{:workflow.id}}"{{/if}}>\n' +
            '  <div class="container-fluid">\n'+
            '    <div class="row-fluid">\n'+
            '      <div class="modal-header"><button type="button" class="close" data-dismiss="modal" aria-hidden="true"><span>x</span></button><h3 id="workflow-content-title" class="force-wrap">{{>title}}</h3></div>\n' +
            '    </div>\n' +
            '    <div class="row-fluid">\n'+
            '      <div class="modal-body">\n' +
            '        <div class="messages"></div>\n' +
            '        <div class="workflow">\n' +
            '          <div class="column span6">'+
            '            <div class="control-group">\n' +
            '              <label class="control-label" for="name">'+swarm.t('Name')+'</label>\n' +
            '              <div class="controls">\n' +
            '                <input class="input-xlarge" type="text" name="name" id="name" {{:mutable ? "" :"disabled=disabled"}} value="{{>workflow.name}}" placeholder="'+swarm.t('Name')+'" required="required">\n' +
            '              </div>\n' +
            '            </div>\n' +
            '            <div class="control-group">\n' +
            '              <label class="control-label" for="description">'+swarm.t('Description')+'</label>\n' +
            '              <div class="controls">\n' +
            '                <textarea class="input-xxlarge" rows="10" name="description" {{:mutable ? "" :"disabled=disabled"}} id="description" placeholder="'+swarm.t('Description')+'">{{>workflow.description}}</textarea>\n' +
            '              </div>\n' +
            '            </div>\n' +
            '            <div class="control-group control-group-owners">\n' +
            '              <label for="owners" class="control-label">'+swarm.t('Owners')+'</label>\n' +
            '              <div class="controls">\n' +
            '                <div class="body in collapse"">\n' +
            '                  <div class="input-prepend" clear="both">\n' +
            '                    <span class="add-on"><i class="icon-user"></i></span>\n' +
            '                    <input type="text" class="input-xlarge multipicker-input{{:mutable ? "" :" disabled"}}" id="owners" data-items="100" data-selected="[]" placeholder="'+swarm.t('Add an Owner')+'">\n' +
            '                  </div>\n' +
            '                  <div class="owners-list multipicker-items-container clearfix"></div>\n' +
            '                </div>\n' +
            '              </div>\n' +
            '            </div>\n' +
            '            <div class="control-group control-group-shared">\n' +
            '              <div class="controls">\n' +
            '                <div class="body in collapse">\n' +
            '                  <input type="hidden" value="0" name="shared">\n' +
            '                  <label class="control-label{{:mutable ? "" :" disabled"}}"><input {{:mutable ? "" :"disabled=disabled"}} type="checkbox" name="shared" id="shared-workflow" {{:workflow.shared ? "checked=checked" : ""}} value="1" />'+swarm.t('Shared with others')+'</label>' +
            '                </div>\n' +
            '              </div>\n' +
            '            </div>\n' +
            '          </div>\n' +
            '          <div class="column span6">'+
            '            <div class="control-group control-group-workflow-rules">\n' +
            '              <h4>'+swarm.t('Rules')+
            '                  <a class="help" href="' + swarm.assetUrl('/docs/Content/Swarm/workflow.overview.html#Workflow_rules') + '" target="_swarm_docs"><span>?</span></a>' +
            '              </h4>\n' +
            '              <div class="panel">\n' +
            '              {{for rules}}' +
            '                <div class="control-group">\n' +
            '                  <label class="rule-description">{{:description}}</label>\n' +
            '                  <div class="btn-group rule">\n' +
            '                    <button class="btn dropdown-toggle dropdown-value" data-toggle="dropdown" data-target="{{:target}}">' +
            '                      <span class="current"></span>' +
            '                      <span class="vertical-bar"></span>' +
            '                      <span class="caret"></span>' +
            '                    </button>\n' +
            '                    <input name="{{:name}}[rule]" type="hidden" value={{:rule}} onError={{:defaultErr}} />\n' +
            '                    <ul class="dropdown-menu" role="menu" aria-label="{{:target}}">\n' +
            '                    {{for options}}' +
            '                      <li><a href="#" data-value="{{:value}}">{{:text}}</a></li>\n' +
            '                    {{/for}}' +
            '                    </ul>' +
            '                  </div>' +
            '                </div>' +
            '              {{/for}}' +
            '              </div>\n' +
            '            </div>\n' +
            '          </div>\n' +
            '        </div>\n' +
            '      </div>\n' +
            '    </div>\n' +
            '    <div class="row-fluid">' +
            '      <div id="affected-projects">' +
            '        <p class="loading animate">'+swarm.t('Loading...')+'</p>' +
            '      </div>' +
            '    </div>\n' +
            '    <div class="row-fluid">\n'+
            '      <div class="modal-footer">\n' +
            '        <div class="control-group group-buttons text-left">\n' +
            '          <div class="controls">\n' +
            '           {{if mutable }}' +
            '            <button type="submit" class="btn btn-mlarge btn-primary btn-save">'+swarm.t('Save')+'</button>\n' +
            '            <button type="button" class="btn btn-mlarge" data-dismiss="modal" aria-hidden="true">'+swarm.t('Cancel')+'</button>\n' +
            '            <div class="tool-tip" data-toggle="tooltip" data-placement="top" title="' + swarm.t('Owners can delete a workflow when not in use') + '">'+
            '              <button disabled="disabled" id="btn-delete" type="button" class="btn btn-mlarge btn-danger" aria-hidden="true">'+swarm.t('Delete')+'</button>\n' +
            '            </div>'+
            '           {{else}}' +
            '            <button type="button" class="btn btn-mlarge btn-primary" data-dismiss="modal" aria-hidden="true">'+swarm.t('Close')+'</button>\n' +
            '           {{/if}}'+
            '          </div>\n' +
            '        </div>\n' +
            '      </div>\n' +
            '    </div>\n' +
            '  </div>\n' +
            '</form>\n';
        swarm.workflow.affectedProjectsTemplate =
            '<ul>' +
            '  {{for projects itemVar="~project"}}' +
            '  <li class="project">' +
            '    <p class="project-name">' +
            '      <a class="collapsed" href="#open-project-{{:#index}}" data-toggle="collapse" data-target="#collapse-project-{{:#index}}">' +
            '        <i class="icon icon-chevron-down"></i><span>{{:name}}</span>' +
            '      </a>' +
            '    </p>' +
            '    <div id="collapse-project-{{:#index}}" class="collapse">' +
            '      <p class="branch-heading">' + swarm.t('Branches') + '</p>' +
            '      <ol class="branch">' +
            '        {{for branches}}' +
            '          {{if (~root.workflowId == workflow || (!workflow && ~root.workflowId == ~project.workflow))}}' +
            '        <li class="branch"><span>{{:name}}</span></li>' +
            '          {{/if}}' +
            '        {{/for}}' +
            '      </ol>' +
            '    </div>' +
            '  </li>' +
            '  {{/for}}' +
            '</ul>\n';
    }
};
